@@ -105,8 +105,8 @@ def fetch_crypto_daily_data(ssj=None, local_root_directory=None, daily_return_pk
         dataframe = pd.read_pickle(pickle_saving_path)
     return dataframe
 
-
-def fetch_crypto_minute_data(ssj=None, pickle_saving_path=None, refetch_all=True):
+def fetch_crypto_minutely_data(ssj=None, local_root_directory=None, minutely_return_pkl_filename_suffix=None,refetch_all=True):
+    pickle_saving_path = local_root_directory + ssj + minutely_return_pkl_filename_suffix
     if refetch_all:
         year = datetime.datetime.now().year
         month = datetime.datetime.now().month
@@ -207,7 +207,9 @@ def fetch_crypto_minute_data(ssj=None, pickle_saving_path=None, refetch_all=True
         dataframe = pd.read_pickle(pickle_saving_path)
     return dataframe
 
-def fetch_crypto_hourly_data(ssj=None, pickle_saving_path=None, refetch_all=True):
+
+def fetch_crypto_hourly_data(ssj=None, local_root_directory=None, hourly_return_pkl_filename_suffix=None,refetch_all=True, daily_crypto_starting_day='2012-01-01'):
+    pickle_saving_path = local_root_directory + ssj + hourly_return_pkl_filename_suffix
     if refetch_all:
         year = datetime.datetime.now().year
         month = datetime.datetime.now().month
@@ -308,6 +310,9 @@ def fetch_crypto_hourly_data(ssj=None, pickle_saving_path=None, refetch_all=True
         dataframe = dataframe.drop(columns=['date'])
         print('size fetched')
         print(dataframe.shape)
+        dataframe = dataframe[dataframe.index >= daily_crypto_starting_day]
+        print('size filtered after '+daily_crypto_starting_day)
+        print(dataframe.shape)
         dataframe.to_pickle(pickle_saving_path)
     else:
         dataframe = pd.read_pickle(pickle_saving_path)
