@@ -9,29 +9,15 @@ from napoleontoolbox.features import features_type
 from napoleontoolbox.file_saver import dropbox_file_saver
 from napoleontoolbox.neural_net import lrs_type
 from napoleontoolbox.neural_net import activations
-
+import math
+import numpy as np
 
 class ParalleLauncher():
-    def __init__(self,drop_token='', dropbox_backup=True, local_root_directory='../data',user='napoleon',  db_path_suffix = '_run.sqlite', meta_features_types=[features_type.FeaturesType.STANDARD_ADVANCED], meta_layers=[[2048,1024,512,256,128,64,32]], convolutions=[0], activations=[activations.ActivationsType.LEAK_0SUB_0MAX0], meta_n=[126], meta_seeds=[0], epochss=[1], ss=[21], meta_n_past_features=[21], utilities=['f_drawdown'], meta_lrs_types = [lrs_type.LrType.CONSTANT],meta_lrs = [0.01],lower_bounds=[0.02],upper_bounds=[0.4], meta_normalize=[True]):
-        # print('Epochs ' + str(epochss))
-        # print('Seed ' + str(meta_seeds))
-        # print('Training period size ' + str(meta_n))
-        # print('Neural net layout ' + str(meta_layers))
-        # print('Stationarize ' + str(meta_stationarize))
-        # print('Normalize ' + str(meta_normalize))
-        # print('Advanced ' + str(meta_advance_features))
-        # print('Signals ' + str(meta_advance_signals))
-        # print('History ' + str(meta_history))
-        # print('Normalized ' + str(meta_normalize))
-        # print('Rebalancing ' + str(ss))
-        # print('Activation ' + str(activations))
-        # print('Convolution' + str(convolutions))
-        # print('Utility' + str(utilities))
-        # print('n past features ' + str(meta_n_past_features))
-        # print('Rebalancing ' + str(ss))
+    def __init__(self,drop_token='', dropbox_backup=True, local_root_directory='../data',user='napoleon',  db_path_suffix = '_run.sqlite', meta_features_types=[features_type.FeaturesType.STANDARD_ADVANCED], meta_layers=[[2048,1024,512,256,128,64,32]], convolutions=[0], activations=[activations.ActivationsType.LEAK_0SUB_0MAX0], meta_n=[126], meta_seeds=[0], epochss=[1], ss=[21], meta_n_past_features=[21], utilities=['f_drawdown'], meta_lrs_types = [lrs_type.LrType.CONSTANT],meta_lrs = [0.01],lower_bounds=[0.02],upper_bounds=[0.4], meta_normalize=[True],table_max_size = 2000):
+        self.table_max_size=table_max_size
         self.args = []
-        self.counter = 1
-
+        self.key_args=[]
+        self.mapping_keys = {}
         for n_past_features in meta_n_past_features:
             for convolution in convolutions:
                 for activation in activations:
@@ -50,8 +36,20 @@ class ParalleLauncher():
                                                                     self.args.append((self,seed, utility, layers, epochs,
                                                                                  n_past_features, n, s, feature_type,
                                                                                  activation, convolution,lr, lrs_type, low_bound, up_bound, normalize))
-                                                                    self.counter = self.counter + 1
-        self.args.sort()
+                                                                    self.key_args.append((seed, utility, layers, epochs,
+                                                                                 n_past_features, n, s, feature_type,
+                                                                                 activation, convolution,lr, lrs_type, low_bound, up_bound, normalize))
+
+        self.key_args.sort()
+        self.counter = 1
+        for me_arg in self.key_args:
+            saving_key = self.create_saving_key(*me_arg)
+            self.mapping_keys[saving_key] = self.counter
+            self.counter = self.counter + 1
+        self.run_number = self.counter - 1
+        print('total number run to execute '+str(self.run_number))
+        self.nb_tables = math.ceil(self.run_number/self.table_max_size)
+
         self.local_root_directory = local_root_directory
         self.user = user
         self.db_path_suffix = db_path_suffix
@@ -59,10 +57,26 @@ class ParalleLauncher():
         self.db_path = self.local_root_directory + self.filename
         self.runs = []
         self.totalRow = 0
-        self.instantiateTable()
+        self.instantiateTables()
         self.empty_runs_to_investigate = []
         self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
 
+    def create_saving_key(self,seed, sup, layers, epochs, n_past_features, n, s, feature_type,activation_type, convolution, lr_start, lr_type, low_bound, up_bound, normalize):
+        meArg = (
+            seed, sup, layers, epochs, n_past_features, n, s, feature_type,
+            activation_type, convolution, lr_start, lr_type, low_bound, up_bound, normalize)
+        meArgList = list(meArg)
+        meArgList = [str(it) for it in meArgList]
+        savingKey = ''.join(meArgList)
+        savingKey = savingKey.replace('[', '')
+        savingKey = savingKey.replace(']', '')
+        savingKey = savingKey.replace(',', '')
+        savingKey = savingKey.replace(' ', '')
+        savingKey = savingKey.replace('.', '')
+        # savingKey = savingKey.replace('_','')
+        savingKey = 'T_' + savingKey
+        return savingKey
+
     def launchParallelPool(self, toRun, use_num_cpu):
         with Pool(processes=use_num_cpu) as pool:
             results = pool.starmap(toRun, self.args)
@@ -75,12 +89,16 @@ class ParalleLauncher():
             results.append(predicted_weights)
         return results
 
-    def instantiateTable(self):
+    def instantiateTables(self):
+        for table_counter in range(self.nb_tables):
+            print('instantiating table number '+str(table_counter))
+            self.instantiateTable(table_counter)
+
+    def instantiateTable(self, table_number):
         sqliteConnection = None
         try:
             sqliteConnection = sqlite3.connect(self.db_path)
-            sqlite_create_table_query = '''CREATE TABLE parallel_run (
-                                        effective_date date PRIMARY KEY);'''
+            sqlite_create_table_query = 'CREATE TABLE parallel_run_'+str(table_number)+' (effective_date date PRIMARY KEY);'
             cursor = sqliteConnection.cursor()
             print("Successfully Connected to SQLite")
             cursor.execute(sqlite_create_table_query)
@@ -94,11 +112,11 @@ class ParalleLauncher():
                 sqliteConnection.close()
                 print("sqlite connection is closed")
 
-    def addRun(self,run):
+    def addRun(self, table_number, run):
         sqliteConnection = None
         try:
             sqliteConnection = sqlite3.connect(self.db_path)
-            sqlite_create_table_query = 'alter table parallel_run add column ' + run + ' real'
+            sqlite_create_table_query = 'alter table parallel_run_'+str(table_number)+' add column ' + run + ' real'
             cursor = sqliteConnection.cursor()
             print("Successfully Connected to SQLite")
             cursor.execute(sqlite_create_table_query)
@@ -113,14 +131,14 @@ class ParalleLauncher():
                 sqliteConnection.close()
                 print("sqlite connection is closed")  # adding the column
 
-    def insertResults(self, run, values):
+    def insertResults(self, table_number, run, values):
         sqliteConnection = None
         try:
             success = True
             sqliteConnection = sqlite3.connect(self.db_path)
             cursor = sqliteConnection.cursor()
             for i, v in values.iteritems():
-                sqlite_insert_query = """INSERT INTO 'parallel_run' ('effective_date', '""" + run + """') VALUES ('""" + str(
+                sqlite_insert_query = """INSERT INTO 'parallel_run_""" +str(table_number) + """' ('effective_date', '""" + run + """') VALUES ('""" + str(
                     i) + """','""" + str(v) + """');"""
                 cursor.execute(sqlite_insert_query)
             sqliteConnection.commit()
@@ -134,13 +152,13 @@ class ParalleLauncher():
                 print("The Sqlite connection is closed")
         return success
 
-    def updateResults(self, run, values):
+    def updateResults(self, table_number, run, values):
         sqliteConnection = None
         try:
             sqliteConnection = sqlite3.connect(self.db_path)
             cursor = sqliteConnection.cursor()
             for i, v in values.iteritems():
-                sqlite_update_query = """UPDATE 'parallel_run' set '""" + run + """' = '""" + str(
+                sqlite_update_query = """UPDATE 'parallel_run_"""+str(table_number)+"""' set '""" + run + """' = '""" + str(
                     v) + """' where effective_date = '""" + str(i) + """'"""
                 cursor.execute(sqlite_update_query)
             sqliteConnection.commit()
@@ -209,27 +227,38 @@ class ParalleLauncher():
                 sqliteConnection.close()
                 print("sqlite connection is closed")
 
-    def addResults(self, run, values):
-        success = self.insertResults(run,values)
+    def addResults(self, table_number, run, values):
+        success = self.insertResults(table_number, run, values)
         if not success :
-            self.updateResults(run, values)
+            self.updateResults(table_number, run, values)
 
+    def getRunAssociatedTable(self, run):
+        run_counter = self.mapping_keys[run]
+        table_number = run_counter % self.nb_tables
+        return table_number
 
     def saveResults(self, run, values, weights_df, recordings_df, activations_df, last_predicted_weights_df):
-        self.addRun(run)
-        self.addResults(run, values)
-        self.totalRow = self.datesCount()
+        table_number = self.getRunAssociatedTable(run)
+        self.addRun(table_number,run)
+        self.addResults(table_number, run, values)
+        #self.totalRow = self.datesCount()
         self.addRunWeights(run, weights_df)
         self.addRunWeightsPrediction(run,last_predicted_weights_df.columns[0], last_predicted_weights_df)
         self.addRunRecordings(run, recordings_df)
         self.addRunActivationsFollowUp(run, activations_df)
 
     def checkRunExistence(self,run):
+        for table_number in range(self.nb_tables):
+            if self.checkLocalRunExistence(table_number,run):
+                return True, table_number
+        return False, np.nan
+
+    def checkLocalRunExistence(self,table_number, run):
         sqliteConnection = None
         runExistence=None
         try:
             sqliteConnection = sqlite3.connect(self.db_path)
-            sqlite_select_run_query = 'SELECT effective_date,'+run + ' FROM parallel_run'
+            sqlite_select_run_query = 'SELECT effective_date,'+run + ' FROM parallel_run_'+str(table_number)
             cursor = sqliteConnection.cursor()
             print("Successfully Connected to SQLite")
             cursor.execute(sqlite_select_run_query)
