@@ -128,7 +128,7 @@ class _RollingBasis:
             self.loss_train += [lo]
         self.handle_callback('after_batch')
         # Set eval and test periods
-        return slice(self.t - self.s, self.t), slice(self.t, self.t + self.s)
+        return slice(self.t - self.s, self.t-1), slice(self.t, self.t + self.s-1)
 
 
 
@@ -254,5 +254,5 @@ class _ShortCutRollingBasis:
             self.loss_train += [lo]
         self.handle_callback('after_batch')
         # Set eval and test periods
-        return slice(self.t - self.s, self.t), slice(self.t, self.t + self.s)
+        return slice(self.t - self.s, self.t-1), slice(self.t, self.t + self.s-1)
 
