@@ -139,7 +139,7 @@ def compute_signals_kpi(data=None, mapping_data=None, print_turnover = False, ke
         for transac_cost in [True, False]:
             temp_df = data[['close', sig]].copy()
             temp_df = temp_df.rename(columns={sig: "signal"}, errors="raise")
-            freqly_df = signal_utility.reconstitute_signal_perf(data=temp_df, transaction_cost=transac_cost,print_turnover=print_turnover)
+            freqly_df, average_turn_over = signal_utility.reconstitute_signal_perf(data=temp_df, transaction_cost=transac_cost,print_turnover=print_turnover)
             sharpe_strat = metrics.sharpe(freqly_df['perf_return'].dropna(), period=252 * 24 * 60, from_ret=True)
             sharpe_under = metrics.sharpe(freqly_df['close_return'].dropna(), period=252 * 24 * 60, from_ret=True)
             kpis.append(
@@ -147,6 +147,7 @@ def compute_signals_kpi(data=None, mapping_data=None, print_turnover = False, ke
                     'sharpe_under': sharpe_under,
                     'sharpe_strat': sharpe_strat,
                     'signal': sig,
+                    'average_turn_over':average_turn_over,
                     'transaction_cost': transac_cost,
                     'parameters_hash': hr_dic[sig]
                 }
@@ -183,7 +184,9 @@ def compute_signals_kpi(data=None, mapping_data=None, print_turnover = False, ke
         me_signals_to_keep = results_dic_without_df['index'].apply(lambda x: x in signals_to_keep)
         results_dic_without_df = results_dic_without_df.loc[me_signals_to_keep]
         kpis_df =kpis_df[kpis_df.signal.isin(list(results_dic_without_df['index'] ))]
-    return kpis_df
+
+    data = data[[col for col in data.columns if col in set(kpis_df.signal) or col == 'close']]
+    return kpis_df, data, hr_dic, json_dic
 
 
 def reconstitute_signal_perf(data=None, initial_price = 1. , average_execution_cost = 7.5e-4 , transaction_cost = True, print_turnover = False, normalization = False):
@@ -191,9 +194,10 @@ def reconstitute_signal_perf(data=None, initial_price = 1. , average_execution_c
         #data.signal = (data.signal - data.signal.mean())/data.signal.std(ddof=0)
         data.signal = expanding_zscore(signal_np_array=data.signal.values)
     data['turn_over'] = abs(data['signal'] - data['signal'].shift(-1).fillna(0.))
+    average_turn_over = data['turn_over'].sum() / len(data)
     if print_turnover:
-        print('average hourly turn over')
-        print(data['turn_over'].sum() / len(data))
+        print('average freqly turn over')
+        print(average_turn_over)
 
     data['close_return'] = data['close'].pct_change()
     data['reconstituted_close'] = metrics.from_ret_to_price(data['close_return'],initial_price=initial_price)
@@ -203,7 +207,7 @@ def reconstitute_signal_perf(data=None, initial_price = 1. , average_execution_c
     else :
         data['perf_return'] = data['non_adjusted_perf_return']
     data['reconstituted_perf'] = metrics.from_ret_to_price(data['perf_return'],initial_price=initial_price)
-    return data
+    return data, average_turn_over
 
 def reconstitute_prediction_perf(y_pred=None, y_true=None, initial_price = 1. , average_execution_cost = 7.5e-4 , transaction_cost = True, print_turnover = False):
     if not isinstance(y_pred, pd.DataFrame):
